#!/bin/sh
#set -x

print_Usage() {
		echo ""
		echo "-------- QLogic Corporation -- Firmaware Dump Utility --------"
		echo ""
		echo "./qlcnic_dump.sh -l ethX|ALL"
		echo "Display firmware dump information on a \"single\" or \"all\" qlcnic adapter"
		echo ""
		echo "./qlcnic_dump.sh -s ethX|ALL"
		echo "Save firmware dump from a \"single\" or \"all\" qlcnic adapter"
		echo "firmware dump will be saved in directory \"/opt/QLogic_Corporation/FW_Dumps\""
		echo "Saved file will be in \"qlcnic_fw_dump_ethX_date_time.txt.gz\" format"
		echo ""
		echo "./qlcnic_dump.sh -f ethX|ALL"
		echo "Force firmware dump on a \"single\" or \"all\" qlcnic adapter"
		echo ""
		echo "./qlcnic_dump.sh -c ethX|ALL 1|0"
		echo "Enable(1)/Disable(0) firmware dump on a \"single\" or \"all\" qlcnic adapter"
		echo ""
		echo "./qlcnic_dump.sh -m ethX|ALL <Mask-Value>"
		echo "Change the current firmware dump capture mask on a \"single\" or \"all\" qlcnic adapter"
		echo "Allowed firmware dump capture mask values are 0x3, 0x7, 0xf, 0x1f, 0x3f, 0x7f, and 0xff"
		echo "Default firmware dump capture mask value is 0x1f"
		echo ""
		echo "where ethX is any interface name belonging to an adapter"
		echo ""
		exit 1
}




Verify_drvr_version(){

	ethtool -i $1 &>/dev/null
	if [ $? -eq 0 ];then
		no_of_dots=`ethtool -i "$1" | grep version | head -1 | cut -d' ' -f2 |grep -o "\." | wc -l`
		no_of_dots=$((no_of_dots+1))
		loop_in_dot=1
	else
		return
	fi
	#echo $no_of_dots
	while [ $loop_in_dot -le $no_of_dots ]
	do
		ethtool -i $1 &>/dev/null
		if [ $? -eq 0 ];then
			figure=`ethtool -i "$1" | grep version | head -1 | cut -d' ' -f2 | cut -d'.' -f"$loop_in_dot"`
		else
			return
		fi
		#echo $figure
		if [ $loop_in_dot -eq 1 ];then
			Major=$figure
		fi

		if [ $loop_in_dot -eq 2 ];then
			Minor=$figure
		fi

		if [ $loop_in_dot -eq 3 ];then
			Subver=$figure
		fi
		loop_in_dot=$((loop_in_dot+1))			
	done

	if [ $Major -gt 5 -o $Minor -gt 0 -o $Subver -gt 25 ];then
		older_driver=0
		version_verified=1
	else
		older_driver=1
		version_verified=1
	fi

	if [ $older_driver -eq 1 ];then
		 echo ""
		 echo "qlcnic: \""qlcnic_dump.sh"\" requires driver version 5.0.26 or higher."
                 echo ""
                 exit
	fi
	#echo "succeed"		
}





err() {
		echo "$@" >&2
		if [ -x /bin/logger ]; then
			/bin/logger -t "${0##*/}[$$]" "$@"
		fi
}

# This Function will find All the interfaces on a single Adapter
Find_interfaces_single() {

		k=0
		m=0
		Flag=`find /sys/devices -name "$ETH" | grep "/net/$ETH"`
		if [ "$Flag" == "" ] ; then
			eth_dir="find /sys/devices -name "net:$ETH""
			Bus=`$eth_dir | awk -F/ '{ for (i = NF; i > 0; --i) print $i }' | head -2 | grep "0000" | cut -d'.' -f1 | cut -d':' -f2,3`
		else
			eth_dir="find /sys/devices -name "$ETH""
			Bus=`$eth_dir | awk -F/ '{ for (i = NF; i > 0; --i) print $i }' | head -3 | grep "0000" | cut -d'.' -f1 | cut -d':' -f2,3`
		fi

		#Check the Bus info with Ethtool if no valid Bus is Determined Corresponding to an interface
		if [ "$Bus" == "" ] ; then
			 Bus=`ethtool -i $ETH | grep bus | cut -d' ' -f 2 | cut -d':' -f2,3 | cut -d'.' -f1`
		fi
		#echo $Bus
		for i in `/sbin/lspci -d1077:| grep Ethernet| cut -d' ' -f 1 | grep "$Bus"`
		do
                        ls /sys/bus/pci/devices/0000\:"$i" | grep net &>/dev/null

                        if [ $? -eq 0 ] ; then
                                                ls /sys/bus/pci/devices/0000\:"$i"/net/ &>/dev/null
                                                if [ $? -eq 0 ] ; then
                                                        Eth=`ls /sys/bus/pci/devices/0000\:"$i"/net/`
							ethtool -i $Eth &>/dev/null
							if [ $? -eq 0 ];then
                                                        	ethtool -i $Eth | grep qlcnic &>/dev/null
                                                        	if [ $? -eq 0 ];then
									dev_parse=1
                                                                	no_of_nodes=`ls /sys/class/net/$Eth/device/ | grep -c "fwdump*"` &>/dev/null
                                                                	if [ $no_of_nodes -gt 1 ];then
                                                                        	brr[m]="0000:"$i""
                                                                        	arr[k]=$Eth
                                                                        	k=$((k+1))
                                                                        	m=$((m+1))
                                                                	fi
                                                        	fi
							fi
                                                else
                                                        Eth=`ls /sys/bus/pci/devices/0000\:"$i"/ | grep net | cut -d':' -f 2`
							ethtool -i $Eth &>/dev/null
							if [ $? -eq 0 ];then
                                                        	ethtool -i $Eth | grep qlcnic &>/dev/null
                                                        	if [ $? -eq 0 ];then
									dev_parse=1
                                                                	no_of_nodes=`ls /sys/class/net/$Eth/device/ | grep -c "fwdump*"` &>/dev/null
                                                                	if [ $no_of_nodes -gt 1 ];then
                                                                        	brr[m]="0000:"$i""
                                                                        	arr[k]=$Eth
                                                                        	k=$((k+1))
                                                                        	m=$((m+1))
                                                                	fi
								fi
                                                        fi
                                                fi
                       fi
		done
}

# This Function will find All the interfaces on All Adapters Present in the System
Find_interfaces_all() {

		k=0
		m=0
		if [ "$option" == "-f" -a "$ETH" == "ALL" ] ; then
			for i in `/sbin/lspci -d1077:| grep Ethernet| cut -d' ' -f 1 | cut -d'.' -f1 | uniq`
			do
				ls /sys/bus/pci/devices/0000\:"$i.0" | grep net &>/dev/null
                                if [ $? -eq 0 ] ; then
                                                ls /sys/bus/pci/devices/0000\:"$i.0"/net/ &>/dev/null
                                                if [ $? -eq 0 ] ; then
                                                        Eth=`ls /sys/bus/pci/devices/0000\:"$i.0"/net/`
							ethtool -i $Eth &>/dev/null
							if [ $? -eq 0 ];then
                                                        	ethtool -i $Eth | grep qlcnic &>/dev/null
                                                        	if [ $? -eq 0 ];then
									Verify_drvr_version $Eth
									dev_parse=1
                                                                	no_of_nodes=`ls /sys/class/net/$Eth/device/ | grep -c "fwdump*"` &>/dev/null
                                                                	if [ $no_of_nodes -gt 1 ];then
										brr[m]="0000:"$i""
                                                                        	arr[k]=$Eth
                                                                        	k=$((k+1))
                                                                        	m=$((m+1))
                                                                	fi
                                                        	fi
							fi
                                                else
                                                        Eth=`ls /sys/bus/pci/devices/0000\:"$i.0"/ | grep net | cut -d':' -f 2`
							ethtool -i $Eth &>/dev/null
							if [ $? -eq 0 ];then
                                                        	ethtool -i $Eth | grep qlcnic &>/dev/null
                                                        	if [ $? -eq 0 ];then
									Verify_drvr_version $Eth
									dev_parse=1
                                                                	no_of_nodes=`ls /sys/class/net/$Eth/device/ | grep -c "fwdump*"` &>/dev/null
                                                                	if [ $no_of_nodes -gt 1 ];then
                                                                        	brr[m]="0000:"$i""
                                                                        	arr[k]=$Eth
                                                                        	k=$((k+1))
                                                                        	m=$((m+1))
                                                                	fi
                                                        	fi
							fi
                                                fi
                               fi

			done
		else
			for i in `/sbin/lspci |grep QLogic | grep Ethernet| cut -d' ' -f 1`
			do
                        	ls /sys/bus/pci/devices/0000\:"$i" | grep net &>/dev/null
                        	if [ $? -eq 0 ] ; then
                                                ls /sys/bus/pci/devices/0000\:"$i"/net/ &>/dev/null
                                                if [ $? -eq 0 ] ; then
                                                        Eth=`ls /sys/bus/pci/devices/0000\:"$i"/net/`
							ethtool -i $Eth &>/dev/null
							if [ $? -eq 0 ];then
                                                        	ethtool -i $Eth | grep qlcnic &>/dev/null
                                                        	if [ $? -eq 0 ];then
									Verify_drvr_version $Eth
									dev_parse=1
                                                                	no_of_nodes=`ls /sys/class/net/$Eth/device/ | grep -c "fwdump*"` &>/dev/null
                                                                	if [ $no_of_nodes -gt 1 ];then
									#	echo $i
                                                                        	brr[m]="0000:"$i""
                                                                        	arr[k]=$Eth
									#	echo ${brr[$m]}
                                                                        	k=$((k+1))
                                                                        	m=$((m+1))
                                                                	fi
                                                        	fi
							fi
                                		else
                                                        Eth=`ls /sys/bus/pci/devices/0000\:"$i"/ | grep net | cut -d':' -f 2`
							ethtool -i $Eth &>/dev/null
							if [ $? -eq 0 ];then
                                                        	ethtool -i $Eth | grep qlcnic &>/dev/null
                                                        	if [ $? -eq 0 ];then
									Verify_drvr_version $Eth
									dev_parse=1
                                                                	no_of_nodes=`ls /sys/class/net/$Eth/device/ | grep -c "fwdump*"` &>/dev/null
                                                                	if [ $no_of_nodes -gt 1 ];then
                                                                        	brr[m]="0000:"$i""
                                                                        	arr[k]=$Eth
                                                                        	k=$((k+1))
                                                                        	m=$((m+1))
                                                                	fi
								fi
                                                        fi
                                                fi
                       	       fi
		       done
		fi

		if [ $version_verified -ne 1 ];then
			echo "qlcnic: Interfaces are temporarily unavailable"
                	echo "Please check the interfaces and try the operation later"
                	echo ""
                	exit
		fi	
}

Save_dump() {
		echo ""
		mkdir -p $DFILE_PATH &> /dev/null

		if [ "$ETH" == "ALL" ] ; then
			Find_interfaces_all
		else
			Find_interfaces_single
		fi
		index=0
		while [ $index -lt ${#arr[@]} ]
		do
			QFWD=${SYSFS}/class/net/${arr[$index]}/device/fw_dump
			QFWDS=${SYSFS}/class/net/${arr[$index]}/device/fwdump_size
			DFILE=${DFILE_PATH}/qlcnic_fw_dump_"${arr[$index]}"_`eval date +%Y%m%d_%H%M%S`.txt

			if ! test -f ${QFWD} ; then
				#err "qlcnic ${arr[$index]}: doesn't support firmware dump capability"
				index=$(( $index + 1 ))
			else
				size=`cat ${QFWDS}`
				if [ "$size" -ne 0 ] ; then
					(( remainder = size%4096 ))
					(( loops = size/4096 ))
					if [ $remainder != "0" ] ; then
						(( loops++ ))
					fi
					for (( i = 0; i < $loops ; i++ ))
					do
						cat ${QFWD} >> ${DFILE}
					done

					if ! test -s "${DFILE}" ; then
						rm ${DFILE}
						index=$(( $index + 1 ))
					else
						gzip ${DFILE}
						err "qlcnic ${arr[$index]}-${brr[$index]}: Firmware dump saved to file "${DFILE}.gz" "
						fw_dump_op=$(( $fw_dump_op + 1 ))
						index=$(( $index + 1 ))
					fi
				else
					#echo "qlcnic ${arr[$index]}: Firmware dump is not available"
					NO_FW_DUMP=$(( $NO_FW_DUMP + 1 ))
					index=$(( $index + 1 ))
				fi
			fi
		done
		echo ""
}

Force_dump() {

			if [ "$ETH" == "ALL" ] ; then
				Find_interfaces_all
				index=0
				#echo  ${#arr[@]}
				while [ $index -lt ${#arr[@]} ]
				do
					ls ${SYSFS}/class/net/${arr[$index]}/device/fw_dump &>/dev/null
					if [ $? -ne 0 ] ; then
						#echo "qlcnic ${arr[$index]}: Does not support firmware dump capability"
						index=$(( $index + 1 ))
					else
						size=`cat /sys/class/net/${arr[$index]}/device/fwdump_size`
						if [ "$size" -ne 0 ] ; then
							echo "qlcnic ${arr[$index]}-${brr[$index]}: Please save the firmware dump first"
							echo "qlcnic ${arr[$index]}-${brr[$index]}: Force firmware dump cannot be performed"
							echo ""
							fw_dump_op=$(( $fw_dump_op + 1 ))
							index=$(( $index + 1 ))
						else
							ls /sys/class/net/${arr[$index]}/device/fwdump_state &>/dev/null
							if [ $? -eq 0 ] ; then
                                                        	DS=`cat /sys/class/net/${arr[$index]}/device/fwdump_state`
                                                        	if [ $DS -ne 1 ] ; then
                                                                	echo ""
                                                                	echo "qlcnic ${arr[$index]}-${brr[$index]}: Firmware dump is not enabled"
                                                                	echo "qlcnic ${arr[$index]}-${brr[$index]}: Force firmware dump cannot be performed"
                                                                	echo ""
                                                                	index=$(( $index + 1 ))
									fw_dump_op=$(( $fw_dump_op + 1 ))
								else
									ethtool -i ${arr[$index]} &>/dev/null
									if [ $? -ne 0 ] ; then
										echo ""
										echo "qlcnic ${arr[$index]}-${brr[$index]}: is not available"
										echo "qlcnic ${arr[$index]}-${brr[$index]}: Force firmware dump cannot be performed"
										echo ""
										fw_dump_op=$(( $fw_dump_op + 1 ))
										index=$(( $index + 1 ))
									else
										echo 0xdeadfeed > /sys/class/net/${arr[$index]}/device/fw_dump
										echo "qlcnic ${arr[$index]}-${brr[$index]}: Force firmware dump performed"
										index=$(( $index + 1 ))
										fw_dump_op=$(( $fw_dump_op + 1 ))
									fi
								fi
							else
								ethtool -i ${arr[$index]} &>/dev/null
								if [ $? -ne 0 ] ; then
									echo ""
									echo "qlcnic ${arr[$index]}-${brr[$index]}: is not available"
									echo "qlcnic ${arr[$index]}-${brr[$index]}: Force firmware dump cannot be performed"
									echo ""
									fw_dump_op=$(( $fw_dump_op + 1 ))
									index=$(( $index + 1 ))
								else
									echo 0xdeadfeed > /sys/class/net/${arr[$index]}/device/fw_dump
									echo "qlcnic ${arr[$index]}-${brr[$index]}: Force firmware dump performed"
									index=$(( $index + 1 ))
									fw_dump_op=$(( $fw_dump_op + 1 ))
								fi
							fi
						fi
					fi
				done
				echo ""
			else
				ethtool -i $ETH &>/dev/null
				if [ "$?" -ne 0 ] ; then
					echo ""
					echo "qlcnic $ETH: is not available"
					echo "Operation aborted"
					echo ""
					exit
				fi

				ls ${SYSFS}/class/net/$ETH/device/fw_dump &>/dev/null
				if [ $? -ne 0 ] ; then
					echo "qlcnic $ETH: No firmware dump interface available!!"
					echo "Operation aborted"
					exit 0
				else
					size=`cat /sys/class/net/$ETH/device/fwdump_size`
					if [ "$size" -ne 0 ] ; then
						echo "qlcnic $ETH: Please save the dump first"
						echo "qlcnic $ETH: Force firmware dump cannot be performed"
						echo ""
						exit 0
					else
						ls /sys/class/net/$ETH/device/fwdump_state &>/dev/null
						if [ $? -eq 0 ] ; then
							DS=`cat /sys/class/net/$ETH/device/fwdump_state`
	                                                if [ $DS -ne 1 ] ; then
                                                        	echo ""
                                                                echo "qlcnic $ETH: Firmware dump is not enabled"
                                                                echo "qlcnic $ETH: Force firmware dump cannot be performed"
                                                                echo ""
								exit 0
                                                        else
								echo 0xdeadfeed > /sys/class/net/$ETH/device/fw_dump
								echo "qlcnic $ETH: Force firmware dump performed"
								exit 0
							fi
						else									
							echo 0xdeadfeed > /sys/class/net/$ETH/device/fw_dump
							echo "qlcnic $ETH: Force firmware dump performed"
							exit 0
						fi
					fi
				fi
			fi
}

Disable_dump() {

			if [ "$ETH" == "ALL" ] ; then
				Find_interfaces_all
			else
				Find_interfaces_single
			fi

			index=0
			while [ $index -lt ${#arr[@]} ]
			do
				ls ${SYSFS}/class/net/${arr[$index]}/device/fw_dump &>/dev/null
				if [ $? -ne 0 ] ; then
					#echo "qlcnic ${arr[$index]}: Does not support firmware dump capability"
					index=$(( $index + 1 ))
				else
					echo 0xbadfeed > ${SYSFS}/class/net/${arr[$index]}/device/fw_dump
					#echo "qlcnic ${arr[$index]}: Firmware dump is disabled"
					index=$(( $index + 1 ))
					fw_dump_op=$(( $fw_dump_op + 1 ))
				fi
			done
			echo ""
}

Enable_dump() {

			if [ "$ETH" == "ALL" ] ; then
				Find_interfaces_all
			else
				Find_interfaces_single
			fi

			index=0
			while [ $index -lt ${#arr[@]} ]
			do
				ls ${SYSFS}/class/net/${arr[$index]}/device/fw_dump &>/dev/null
				if [ $? -ne 0 ] ; then
					index=$(( $index + 1 ))
				else
					echo 0xaddfeed > ${SYSFS}/class/net/${arr[$index]}/device/fw_dump
					#echo "qlcnic ${arr[$index]}: Firmware dump is enabled"
					fw_dump_op=$(( $fw_dump_op + 1 ))
					index=$(( $index + 1 ))
				fi
			done
			echo ""
}


Change_mask() {
			if [ "$ETH" == "ALL" ] ; then
				Find_interfaces_all
			else
				Find_interfaces_single
                        fi
			index=0
			while [ $index -lt ${#arr[@]} ]
			do
				ls ${SYSFS}/class/net/${arr[$index]}/device/fwdump_level &>/dev/null
				if [ $? -ne 0 ] ; then
					#echo "qlcnic ${arr[$index]}: Does not support firmware dump capability"
					index=$(( $index + 1 ))
				else
					mask=`printf "%d\n" ${MASK}`
					case $mask in
					3) ;;
					7) ;;
				       15) ;;
				       31) ;;
				       63) ;;
				      127) ;;
			              255) ;;
					*) echo "qlcnic: Invalid capture mask "\"$MASK"\""
					   echo "Operation aborted"
					   echo ""
					   exit
					   ;;
					esac
					echo ${MASK} > "${SYSFS}/class/net/${arr[$index]}/device/fwdump_level"
					#echo "qlcnic ${arr[$index]}: Firmware dump capture mask changed to: \"${MASK}\""
					index=$(( $index + 1 ))
					fw_dump_op=$(( $fw_dump_op + 1 ))
				fi
			done
}

Display() {

			if [ "$ETH" == "ALL" ] ; then
				Find_interfaces_all
			else
				Find_interfaces_single
			fi

			index1=0
			index=0
			while [ $index -lt ${#arr[@]} ]
			do
				ls ${SYSFS}/class/net/${arr[$index]}/device/fw_dump &>/dev/null
				if [ $? -ne 0 ] ; then
					#echo "qlcnic ${arr[$index]}: Does not support firmware dump capability"
					index=$(( $index + 1 ))
					index1=$(( $index1 + 1 ))
				else
				#	echo ${arr[$index]}
					PROP1=`cat ${SYSFS}/class/net/${arr[$index]}/device/fwdump_level`
					PROP1=`printf "%x\n" $PROP1`
					PROP2=`cat ${SYSFS}/class/net/${arr[$index]}/device/fwdump_size`

                                        ls ${SYSFS}/class/net/${arr[$index]}/device/fwdump_state &>/dev/null
                                        if [ $? -eq 0 ];then
                                                PROP3=`cat ${SYSFS}/class/net/${arr[$index]}/device/fwdump_state`
                                                if [ $PROP3 -ne 0 ] ; then
                                                        flag1="Yes"
                                                else
                                                        flag1="No"
                                                fi

                                        else
                                                flag1="Not Supported"
                                        fi

					size=`cat ${SYSFS}/class/net/${arr[$index]}/device/fwdump_size`
					if [ $PROP2 -ne 0 ] ; then
						flag="Yes"
					else
						flag="No"
					fi

					echo ""
					echo "qlcnic: ${arr[$index]}: ${brr[$index]}"
					echo "Firmware dump available		:$flag"
					echo "Firmware dump size		:$size bytes"
					echo "Firmware dump enabled		:$flag1"
					echo "Firmware dump capture mask	:0x$PROP1"
					echo ""
					index=$(( $index + 1 ))
					index1=$(( $index1 + 1 ))
					fw_dump_op=$(( $fw_dump_op + 1 ))
				fi
			done
}

Verify_args(){

	if [ "$1" == "ALL" ];then
		return
	else
		ethtool -i "$1" &>/dev/null
		if [ $? -ne 0 ];then
			echo "qlcnic: Invalid argument \""$1"\""
			echo "Please check the interface \""$1"\" is available or not"
			echo "Operation aborted"
			print_Usage
		else
			ethtool -i "$1" | grep qlcnic &>/dev/null
			if [ $? -ne 0 ];then
				echo "qlcnic: Invalid argument \""$1"\""
				echo "Please pass valid qlcnic interface name"
				echo "Operation aborted"
				print_Usage
			else
				return
			fi
		fi
	fi
}


option=$1
ETH=$2
MASK=$3
SYSFS=/sys
DFILE_PATH=/opt/QLogic_Corporation/FW_Dumps
NOA=$#
QFWD1=${SYSFS}/class/net/$ETH/device/fwdump_level
option_verify=0
fw_dump_op=0
NO_FW_DUMP=0
dev_parse=0
udev_rules=0
udev_script=0
version_verified=0

# Declare An Array in which All the interface Names will be stored
declare -a arr
declare -a brr

if [ "$#" == 0 ] ; then
	print_Usage
fi

lsmod | grep qlcnic &>/dev/null
if [ $? -ne 0 ];then
	echo "qlcnic: Please load the qlcnic driver first"
	echo "Operation aborted"
	exit
fi

if [ "$option" == "-l" -o "$option" == "-f" -o "$option" == "-s" ];then
        option_verify=1
        if [ $NOA -ne 2 ];then
                echo "qlcnic: Please pass valid number of arguments"
		echo "Operation aborted"
                print_Usage
        fi
fi

if [ "$option" == "-c" -o "$option" == "-m" ];then
         option_verify=1
         if [ $NOA -ne 3 ];then
                echo "qlcnic: Please pass valid number of arguments"
		echo "Operation aborted"
                print_Usage
        fi
fi

if [ "$option" == "-h" -o "$option" == "--help" ];then
        print_Usage
	exit
fi

if [ $option_verify -ne 1 ];then
        echo "qlcnic: Please use valid options"
	echo "Operation aborted"
        print_Usage
fi

Verify_args $ETH

if [ "$ETH" != "ALL" ];then
	Verify_drvr_version $ETH

	if [ $version_verified -ne 1 ];then
		echo "qlcnic: Interfaces are temporarily unavailable"
		echo "Please check the interface \"$ETH\" and try the operation later"
		echo ""
		exit
	fi
fi

case "$option" in
-l)
	echo ""
	echo "** qlcnic: Displaying firmware dump information"
	echo ""
	Display

	if [ $dev_parse -eq 0 ];then
		echo "qlcnic: No device interfaces available"
		echo "Operation aborted"
		echo ""
		exit
	fi

	if [ $fw_dump_op -eq 0 ];then
		echo "qlcnic: No firmware dump interface available!!"
		echo "Operation aborted"
		echo ""
		exit
	fi
	exit 0
	;;
-s)
	echo ""
	echo "** qlcnic: Saving the firmware dump **"
	echo ""

	ls /lib/udev/qlcnic_udev.sh &>/dev/null
	if [ $? -eq 0 ];then
		udev_script=1
	fi

	ls /etc/udev/rules.d/99-qlcnic.rules &>/dev/null
	if [ $? -eq 0 ];then
		udev_rules=1
	fi

	if [ $udev_rules -eq 1 -a $udev_script -eq 1 ];then
		echo "qlcnic: udev script \"/lib/udev/qlcnic_udev.sh\" will save the dump automatically"
		echo "Operation aborted"
		echo ""
		exit
	fi

	Save_dump

        if [ $dev_parse -eq 0 ];then
                echo "qlcnic: No device interfaces available"
		echo "Operation aborted"
x
		echo ""
                exit
        fi

	if [ $NO_FW_DUMP -ne 0 -a $fw_dump_op -eq 0 ];then
		echo "qlcnic: firmware dump is not available!!"
		echo "Operation aborted"
		echo ""
		exit
	fi

	if [ $NO_FW_DUMP -eq 0 -a $fw_dump_op -eq 0 ];then
		echo "qlcnic: No firmware dump interface available!!"
		echo "Operation aborted"
		echo ""
		exit
	fi
			
	exit 0
	;;
-f)
	echo ""
	echo "** qlcnic: Forcing the firmware dump **"
	echo ""
	Force_dump

        if [ $dev_parse -eq 0 ];then
                echo "qlcnic: No device interfaces available"
		echo "Operation aborted"
		echo ""
                exit
        fi

	if [ $fw_dump_op -eq 0 ];then
		echo "qlcnic: No firmware dump interface available!!"
		echo "Operation aborted"
		echo ""
	fi

	exit 0
	;;
-c)
	if [ "$3" == "0" -o "$3" == "1" ] ; then
		if [ $3 -ne 0 ] ; then
			echo ""
			Enable_dump
		else
			echo ""
			Disable_dump
		fi
	else
		echo ""
		echo "qlcnic: Please check the argument $3"
		echo "Operation aborted"
		print_Usage
	fi

	if [ $dev_parse -eq 0 ];then
		echo "qlcnic: No device interfaces available"
		echo "Operation aborted"
		echo ""
		exit
	fi

        if [ $fw_dump_op -eq 0 ];then
                echo "qlcnic: No firmware dump interface available!!"
                echo "Operation aborted"
		echo ""
	else
		if [ $3 -ne 0 ];then
			echo "qlcnic: firmware dump is enabled"
			echo ""
		else
			echo "qlcnic: firmware dump is disabled"
			echo ""
		fi
        fi

	exit 0
	;;
-m)
	echo ""
	echo "** qlcnic: Changing the firmware dump capture mask value **"
	echo ""
	Change_mask

        if [ $dev_parse -eq 0 ];then
                echo "qlcnic: No device interfaces available"
		echo "Operation aborted"
		echo ""
                exit
        fi

        if [ $fw_dump_op -eq 0 ];then
                echo "qlcnic: No firmware dump interface available!!"
                echo "Operation aborted"
		echo ""
	else
		echo "qlcnic: Mask changed to \""${MASK}"\""
		echo ""
        fi

	exit 0
	;;
*)
	print_Usage
	;;
esac



